#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
##############################################################################
#
#    Acrisel LTD
#    Copyright (C) 2008- Acrisel (acrisel.com) . All Rights Reserved
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

import os
import pprint
import filecmp
import difflib
import sys
import re
import itertools
from collections import namedtuple

DiffContent=namedtuple('DiffShellowContent', ['diff', 'folders_only_in_dir1', 'folders_only_in_dir2', 'files_only_in_dir1', 'files_only_in_dir2', 'diff_files', 'diff_detail'])

def check_match(path, regs=None, on_empty_ignore=True):
    ''' Return True if any of parts of path matches any regexp in ignore_re
    
        if ignore_re is empty, return on_empty_ignore
        
        Args:
            path: path to file or directory
            ignore_re: list of compiled regular expressions
    '''
    result=on_empty_ignore
    if regs:
        result=False
        parts=path.split(os.sep)
        for reg, part in itertools.product(regs, parts):
            matching=reg.match(part)
            matching=matching.group() if matching else None
            result=matching is not None
            if result: break
    return result

def print_block(block, top_sep=None, bottom_sep='-'):
    if not isinstance(block, list): block=[block]
    
    maxsize=max(map(lambda x: len(x), block))
    if top_sep is not None:
        print(top_sep * maxsize)
    for line in block:
        print(line)
    if bottom_sep is not None:
        print(bottom_sep * maxsize)

def get_artifacts(loc, ignore, followlinks):
    files=list()
    dirs=list()
    if not loc.endswith(os.sep): loc+=os.sep
    
    regs=list(map(lambda x: re.compile(x), ignore)) if len(ignore) > 0 else None 
    
    for dirpath, dirnames, filenames in os.walk(loc, followlinks=followlinks):
        relpath=dirpath[len(loc):]
        for dirname in dirnames: 
            name=os.path.join(relpath, dirname)
            ignore=check_match(name, regs, on_empty_ignore=False)
            if not ignore: dirs.append(name)
        for filename in filenames: 
            name=os.path.join(relpath, filename)
            ignore=check_match(name, regs, on_empty_ignore=False)
            if not ignore: files.append(name)
    return dirs, files

def print_file_diff_header(file, file1, file2):
    head=['diff of %s' % file, "file1: %s" % file1, 'file2: %s' % file2]
    print_block(head)

def commdir(dir1, dir2, ignore=[], detailed=False, followlinks=False, quiet=False, bool_result=True):
    ''' compares two directory structures and their files.
    
    commdir walks through two directories, dir1 and dir2. While walking, it aggregates information
    on the difference between the two structures and their content.
    
    If bool_result is True, commdir will return True if difference was found. 
    When False, it would return a DiffContent namedtuple with the following fields:
    
        - diff (boolean)
        - folders_only_in_dir1 (list)
        - folders_only_in_dir2 (list) 
        - files_only_in_dir1 (list)
        - files_only_in_dir2 (list) 
        - diff_files (list)
        - diff_detail (list)
     
    Args:
        dir1, dir2: two directories structure to compare.
        ignore: list of regular expression strings to ignore, when directory is ignored, all its sub folders are ignored too.
        detailed: if set, will generate detailed file level comparison.
        followlinks: if set, symbolic links will be followed.
        quiet: if set, information will not be printed to stdio.
        bool_result: instruct how the function would respond to caller (True: boolean or False: DiffContent)
        
    '''
    result=False
    
    # create inventory of object to compare:
    dir1_folders, dir1_files = get_artifacts(dir1, ignore=ignore, followlinks=followlinks)
    dir2_folders, dir2_files = get_artifacts(dir2, ignore=ignore, followlinks=followlinks)
    
    folders_only_in_1=sorted(list(set(dir1_folders) - set(dir2_folders) ) )
    if len(folders_only_in_1) > 0:
        result=True
        if not quiet:
            print_block('folders only in %s' % (dir1,), top_sep='-')
            for x in folders_only_in_1: print("  ", x)
            
    folders_only_in_2=sorted(list(set(dir2_folders) - set(dir1_folders) ))
    if len(folders_only_in_2) > 0:
        result=True
        if not quiet:
            print_block('folders only in %s' % (dir2,), top_sep='-')
            for x in folders_only_in_2: print("  ", x)
    
    files_only_in_1=sorted(list(set(dir1_files) - set(dir2_files) ) )
    if len(files_only_in_1) > 0:
        result=True
        if not quiet:
            print_block('files only in %s' % (dir1,), top_sep='-')
            for x in files_only_in_1: print("  ", x)
            
    files_only_in_2=sorted(list(set(dir2_files) - set(dir1_files) ))
    if len(files_only_in_2) > 0:
        result=True
        if not quiet:
            print_block('files only in %s' % (dir2,), top_sep='-')
            for x in files_only_in_2: print("  ", x)
    
    files_in_both=sorted(list(set(dir1_files) & set(dir2_files) ) )
    files_diff=list()
    for file in files_in_both:
        file1=os.path.join(dir1, file)
        file2=os.path.join(dir2, file)
        cmp=filecmp.cmp(file1, file2)
        if not cmp:
            files_diff.append((file, file1, file2))
    if len(files_diff) > 0:
        result=True
        diff_files=files_diff
        if not quiet:
            print_block('files different:', top_sep='-')
            for x in files_diff: print('  ',x[0])
    
    diff_detail=None
    if detailed:            
        for file, file1, file2 in files_diff:
            if not quiet:
                print_file_diff_header(file, file1, file2)
                with open(file1,'r') as f1, open(file2,'r') as f2:
                    try:
                        diff = difflib.ndiff(f1.readlines(),f2.readlines())  
                    except Exception as e:
                        print('Error: failed to diff: %s' % (repr(e), ))
                        continue
                    for i, line in enumerate(diff):
                        if line.startswith(' '): continue
                        sys.stdout.write(str(i) + ': ' +line)
            
    if bool_result:
        return result
    else:
        content=DiffContent(diff=result,
                            folders_only_in_dir1=folders_only_in_1, 
                            folders_only_in_dir2=folders_only_in_2,
                            files_only_in_dir1=files_only_in_1, 
                            files_only_in_dir2=files_only_in_2,
                            diff_files=[x[0] for x in diff_files], 
                            diff_detail=diff_detail)
        return content

def add_os_sep(x):
    if x.endswith(os.sep):
        return x
    else:
        return x+os.sep
    
def same_path_prefix(path1, path2, top=True):
    p1=add_os_sep(path1)
    p2=add_os_sep(path2)
    if p1.startswith(p2):
        return path2 if top else path1
    elif p2.startswith(p1):
        return path1 if top else path2
    else:
        return None
    
def reduce_folder(folders, top=True):
    '''
    take folders and reduce to the smallest prefix that would remove all
    '''
    to_folders=list()
    current_prefix=folders[0]
    
    for folder in folders:
        prefix=same_path_prefix(current_prefix, folder, top=top)
        if prefix:
            current_prefix=prefix
        else:
            to_folders.append(current_prefix)
            current_prefix=folder
            
    to_folders.append(current_prefix)
    return sorted(set(to_folders))
        
def sync_dirs(loc, missing_folders, cmd="mkdir -fp", msg="Create missing folders", quiet=False, top=True):    
    if len(missing_folders) > 0:
        if not quiet: print_block('%s %s:' % (msg, loc,), top_sep='-')
        missing_folders=reduce_folder(missing_folders, top=top)
        for x in missing_folders: 
            if not quiet: print("  %s %s" % (cmd, os.path.join(loc, x)))

def sync_files(source, target, missing_files, quiet=False):    
    if len(missing_files) > 0:
        if not quiet: print_block('Create missing %s files:' % (target,), top_sep='-')
        for x in missing_files: 
            if not quiet: print("  cp -f %s %s" % (os.path.join(source, x), os.path.join(target, x)))

def rm_files(loc, extra_files, quiet=False):    
    if len(extra_files) > 0:
        if not quiet: print_block('Remove extra %s files:' % (loc,), top_sep='-')
        for x in extra_files: 
            if not quiet: print("  rm -f %s" % (os.path.join(loc, x), ))

def merge_files(source, target, diff_files, merge='cp -f', quiet=False):    
    if len(diff_files) > 0:
        if not quiet: print_block('Merge different files:', top_sep='-')
        for x in diff_files: 
            if not quiet: print("  %s %s %s" % (merge, os.path.join(source, x), os.path.join(target, x)))
    
def show_sync_cmd(dir1, dir2, content, merge=False, quiet=False):
    folders_only_in_dir1=content.folders_only_in_dir1
    folders_only_in_dir2=content.folders_only_in_dir2
    files_only_in_dir1=content.files_only_in_dir1
    files_only_in_dir2=content.files_only_in_dir2
    diff_files=content.diff_files
    
    sync_dirs(dir2, folders_only_in_dir1, cmd="mkdir -fp", msg="Create missing folders", quiet=quiet, top=False)
    if merge: 
        sync_dirs(dir1, folders_only_in_dir2, cmd="rmdir -fr", msg="Remove extra folders", quiet=quiet, top=True)
    else:
        sync_dirs(dir2, folders_only_in_dir2, cmd="rmdir -fr", msg="Remove extra folders", quiet=quiet, top=True)
    
    sync_files(dir1, dir2, files_only_in_dir1, quiet=quiet)
    if merge: 
        rm_files(dir2, files_only_in_dir2, quiet=quiet)
    else:
        rm_files(dir2, files_only_in_dir2, quiet=quiet)
    
    merge_cmd='cp -f' if not merge else "diffmerge"
    merge_files(dir1, dir2, diff_files, merge=merge_cmd, quiet=quiet)
    
def show_summary(dir1, sir2, content, quiet=False):
    if not quiet:
        print_block('Summary:', top_sep='-')
        print("  Folders only in %s: %s" % (dir1, len(content.folders_only_in_dir1), ) )
        print("  Files only in %s: %s" % (dir1, len(content.files_only_in_dir1), ) )
        print("  Folders only in %s: %s" % (dir2, len(content.folders_only_in_dir2), ) )
        print("  Files only in %s: %s" % (dir2, len(content.files_only_in_dir2), ) )
        print("  Files different files: %s" % (len(content.diff_files), ) )
    

args_description='''
Reports differences in directory structure and content.

commdir.py will exit with 0 if directories found the same.
otherwise, it will exit with 1.
'''
args_epilog='''
example:
    python commdir.py --dir1 my_folder --dir2 other_folder --ignore __pycache__ .*DS_Store
'''

if __name__ == '__main__':
    import argparse
    
    parser = argparse.ArgumentParser(description=args_description, epilog=args_epilog)
    parser.add_argument('--dir1', metavar='DIR1', type=str, nargs=1,
                    help='source folder for the comparison')
    parser.add_argument('--dir2', metavar='DIR2', type=str, nargs=1, 
                    help='target folder for the comparison')
    parser.add_argument('--quiet', dest='quiet', action='store_true',
                    help='avoid writing any report out, default: False')
    parser.add_argument('--out', metavar='REPORT', dest='out', nargs='?',
                    help='file to write report to, default: stdout')
    parser.add_argument('--follow', action='store_true',
                    help='follow links when walking folders, default: False')
    parser.add_argument('--detailed', action='store_true',
                    help='provide detailed file level diff, default: False')
    parser.add_argument('--sync-cmd', dest='sync_cmd', action='store_true',
                    help='provide commands that would align dirs and files, default: False')
    parser.add_argument('--merge', dest='merge', action='store_true',
                    help='when sync-cmd, set how diff commands would be resolved, default: dir1 is base.')
    parser.add_argument('--total', dest='total', action='store_true',
                    help='outputs summary.')
    parser.add_argument('--ignore', metavar='PATTERN', dest='ignore', type=str, nargs='*',
                    default=list(),
                    help='pattern to ignore')
    
    args = parser.parse_args()
    dir1=args.dir1[0]
    dir2=args.dir2[0]
    quiet=args.quiet
    
    # TODO: handle args.out

    result=commdir(dir1, dir2, ignore=args.ignore, detailed=args.detailed, followlinks=args.follow, quiet=quiet, bool_result=False)
    
    if args.sync_cmd:
        show_sync_cmd(dir1, dir2, result, merge=args.merge, quiet=quiet)
        
    if args.total:
        show_summary(dir1, dir2, result, quiet=quiet)
    
    diff=result.diff
    if not quiet: 
        msg='%s and %s are %s' % (dir1, dir2, 'same' if not diff else 'different')
        print(msg)
    
    exit(0 if not result else 1)
    
    



#!/bin/env python3 
# -*- encoding: utf-8 -*-
##############################################################################
#
#    Acrisel LTD
#    Copyright (C) 2008- Acrisel (acrisel.com) . All Rights Reserved
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from pexpect import pxssh
import getpass
import os
from multiprocessing import Pool
from tempfile import NamedTemporaryFile

def mssh1(commands, host, username, password, sudouser, output):
    out=open(output, "w")
    # open ssh channels
    s = pxssh.pxssh()
    try:
        s.login(host, username, password)
    except pxssh.ExceptionPxssh as e:
        out.write("{}: pxssh failed on login.".format(host))
        out.write("{}: {}".format(host ,e))
        return 

    prefix=''
    if sudouser:
      prefix='sudo -u {} '.format(sudouser)
    
    for command in commands:
        cmd=prefix+command
        #print('send command', cmd)
        s.sendline(cmd)        
        if sudouser:
            try:
                i=s.expect(['.*[pP]assword:.*', s.PROMPT])
            except Exception as e:
                out.write("{}: {}".format(host, s.before))
                out.write("{}: Failed to expect".format(host))
                print("Failed to expect", e)
                out.close()
                s.logout()
                return
            #print("i:", i)
            if i == 0:
                s.sendline(password)
            else:
                pass
        else:
            s.prompt()
        #print("{}: {}".format(host, s.before.decode()))
        out.write("{}: {}".format(host, s.before.decode()))
    out.close()
    s.logout()    

def mssh(commands, parallel, hosts, username, password, sudouser, keeplog):
    with Pool(processes=parallel) as pool: 
        output=[]
        for host in hosts:
            out=NamedTemporaryFile(mode="w", prefix="bee_"+host+"_", suffix=".log", delete=False)
            output.append(out.name)
            out.close()
            if keeplog: print("{} log: {}".format(host, out.name))
            result = pool.apply_async(mssh1, (commands, host, username, password, sudouser, out.name))    
        pool.close()
        pool.join()
        for out in output:
            with open(out, "r") as o:
                print(o.read())
            if not keeplog: os.remove(out)

if __name__ == '__main__':
    from argparse import ArgumentParser 
   
    parser = ArgumentParser(description="""
Sends ssh command to multiple destinations.
""")
    parser.add_argument('-c', '--command', required=True, action='append', metavar='COMMAND', default=[], dest='commands',
                        help="""command to execute over ssh channel""")
    parser.add_argument('-p', '--parallel', type=int, required=False, metavar='PARALLEL', dest='parallel', default=1,
                        help="""number of parallel session to open""")    
    parser.add_argument('-t', '--target', required=True, action='append', metavar='HOST', default=[], dest='hosts',
                        help="""destination host to run against""")
    parser.add_argument('-u', '--user', type=str, required=False, metavar='USERNAME', dest='username', default=os.getenv('USER'),
                        help="""user to use for ssh authentication""")
    parser.add_argument('--sudo-user', type=str, required=False, metavar='USERNAME', dest='sudouser', 
                        help="""sudo user to use to run commands""")
    parser.add_argument('--keep-log', required=False, action='store_true', dest='keeplog',  
                        help="""indicates bee to keep host logs instead of deleting""")

    args = parser.parse_args()
    
    password = getpass.getpass('password: ')
    mssh(commands=args.commands, parallel=args.parallel, hosts=args.hosts, username=args.username, password=password, sudouser=args.sudouser, keeplog=args.keeplog) 



#!/bin/env python3
# -*- encoding: utf-8 -*-
##############################################################################
#
#    Acrisel LTD
#    Copyright (C) 2008- Acrisel (acrisel.com) . All Rights Reserved
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

import argparse
import csv
import sys
import os
from openpyxl.workbook import Workbook

parser = argparse.ArgumentParser(description='Creates Excel file from one or more CSV files.  If multiple CSV are provided, they wiull be mapped to separated sheets.  If "-" is provided, input will be acquire from stdin.')
parser.add_argument("input_files", metavar='CSV', nargs='+', help="csv files to merge in xlsx; if -, stdin is assumed")
parser.add_argument("-d", "--delimiter", metavar='DELIMITER', default=",", help="select delimiter character")
parser.add_argument("-o", "--out", metavar='OUTFILE', help="output xlsx filename")
args = parser.parse_args()


wb = Workbook()
first_sheet=True


for input_file in args.input_files:
    if input_file == '-':
        input_fh=sys.stdin
        sheet_name='sheet'
    elif not input_file.endswith(".csv"):
        sys.stderr.write("Error: File does not have the ending \".csv\".\n")
        sys.exit(2)
    else:
        input_fh=open(input_file)
        sheet_name=os.path.basename(input_file).rpartition('.')[0]

    if first_sheet:
        sheet = wb.active
        sheet.title=sheet_name
        first_sheet=False
    else:
        sheet = wb.create_sheet(sheet_name)

    for row_index, row in enumerate(
        csv.reader(input_fh, delimiter=args.delimiter), start=1):
        for col_index, col in enumerate(row, start=1):
            sheet.cell(row = row_index, column = col_index).value = col

wb.save(args.out )



#!/bin/env python3
# -*- encoding: utf-8 -*-
##############################################################################
#
#    Acrisel LTD
#    Copyright (C) 2008- Acrisel (acrisel.com) . All Rights Reserved
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

import xml.dom.minidom
import argparse
import sys

parser = argparse.ArgumentParser(description='Pretty prints XML file that is not pretty.')
parser.add_argument("input_files", metavar='XML', nargs='*', help="XML files to pretty print; if - or none provided, stdin is assumed")
parser.add_argument("-o", "--out", metavar='OUTFILE',help="output filename; defaults to stdout")
args = parser.parse_args()

output_fh=sys.stdout
if args.out:
    output_fh=open(args.out, 'w')

input_files=args.input_files
if not input_files:
    input_files=['-']
    
for input_file in input_files:
    if input_file == '-':
        input_fh=sys.stdin
    else:
        input_fh=open(input_file)

    xml_string=input_fh.read()

    if input_file != '-':
        input_fh.close()
        
    # xml = xml.dom.minidom.parse(xml_fname) # by file name
    xml_data=xml.dom.minidom.parseString(xml_string) # by string
    pretty_xml_as_string = xml_data.toprettyxml(indent='    ')

    output_fh.write(pretty_xml_as_string)

if args.out:
    output_fh.close()



__version__ = "2.0.2"

from .idioms.threaded import Threaded, RetriveAsycValue, threaded
from .idioms.singleton import Singleton, NamedSingleton
from .idioms.sequence import Sequence
from .idioms.timed_sized_logging_handler import TimedSizedRotatingHandler
from .idioms.mplogger import MpLogger, LevelBasedFormatter, create_stream_handler
from .idioms.data_types import MergedChainedDict
import acris.idioms.resource_pool as resource_pool
import acris.idioms.virtual_resource_pool as virtual_resource_pool
from .idioms.resource_pool import ResourcePool, Resource, Requestor, Requestors
from acris.idioms.synchronize import Synchronization, SynchronizeAll, dont_synchronize
from acris.idioms import synchronize
from .idioms.mediator import Mediator
from acris.idioms.decorate import traced_method


#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
##############################################################################
#
#    Acrisel LTD
#    Copyright (C) 2008- Acrisel (acrisel.com) . All Rights Reserved
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

"""Send the contents of a directory as a MIME message."""

import os
import sys
import smtplib
# For guessing MIME type based on file name extension
import mimetypes

from argparse import ArgumentParser

from email import encoders
from email.message import Message
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

COMMASPACE = ', '

def attach(outer, path):
    # Guess the content type based on the file's extension.  Encoding
    # will be ignored, although we should check for simple things like
    # gzip'd or compressed files.
    ctype, encoding = mimetypes.guess_type(path)
    if ctype is None or encoding is not None:
        # No guess could be made, or the file is encoded (compressed), so
        # use a generic bag-of-bits type.
        ctype = 'application/octet-stream'
    maintype, subtype = ctype.split('/', 1)
    if maintype == 'text':
        with open(path) as fp:
            # Note: we should handle calculating the charset
            msg = MIMEText(fp.read(), _subtype=subtype)
    elif maintype == 'image':
        with open(path, 'rb') as fp:
            msg = MIMEImage(fp.read(), _subtype=subtype)
    elif maintype == 'audio':
        with open(path, 'rb') as fp:
            msg = MIMEAudio(fp.read(), _subtype=subtype)
    else:
        with open(path, 'rb') as fp:
            msg = MIMEBase(maintype, subtype)
            msg.set_payload(fp.read())
        # Encode the payload using Base64
        encoders.encode_base64(msg)
    # Set the filename parameter
    msg.add_header('Content-Disposition', 'attachment', filename=path)
    outer.attach(msg)


def send_mail(mailto, subject, body='', mailfrom=[], mailcc=[], attachments=[], output=''):
    
    # Create the enclosing (outer) message
    outer = MIMEMultipart()
    if not subject:
        subject = 'Content of directory'
        
    outer['Subject'] = subject
    if mailcc:
      outer['Cc'] = COMMASPACE.join(mailcc)
      
    outer['To'] = COMMASPACE.join(mailto)
    
    if not mailfrom:
        mailfrom="{}@{}".format(os.environ['USER'], os.environ['HOSTNAME'])
    outer['From'] = mailfrom

    outer.preamble = 'You will not see this in a MIME-aware mail reader.\n'
    if body:
        outer.attach(MIMEText(body))
        
    for attachment in attachments: 
        if not os.path.isfile(attachment):
            for filename in os.listdir(attachment):
                path = os.path.join(attachment, filename)
                if not os.path.isfile(path):
                    continue
                attach(outer, path)
        else:
            attach(outer, attachment)
        
    # Now send or store the message
    composed = outer.as_string()
    if output:
        with open(output, 'w') as fp:
            fp.write(composed)
    else:
        with smtplib.SMTP('localhost') as s:
            s.send_message(outer)


if __name__ == '__main__':
    parser = ArgumentParser(description="""\
Send the contents of a directory as a MIME message.
Unless the -o option is given, the email is sent by forwarding to your local
SMTP server, which then does the normal delivery process.  Your local machine
must be running an SMTP server.
""")
    parser.add_argument('-a', '--attach', required=False, 
                        action='append', metavar='ATTACHMENT', default=[], dest='attachments',
                        help="""Mail the contents of the specified directory or file, Only the regular
                        files in the directory are sent, and we don't recurse to
                        subdirectories.""")
    parser.add_argument('-o', '--output', metavar='FILE',
                        help="""Print the composed message to FILE instead of
                        sending the message to the SMTP server.""")
    parser.add_argument('-s', '--subject', metavar='SUBJECT', required=True, 
                        help="""Subject for email message (required).""")
    parser.add_argument('-b', '--body', metavar='BODY', required=False,
                        help="""Boby text for the message (optional).""")
    parser.add_argument('-f', '--mailfrom', required=False,
                        help='The value of the From: header (optional); if not provided $USER@$HOSTNAME will be use as sender')
    parser.add_argument('-c', '--malicc', required=False,
                        help='The value of the CC: header (optional)', action='append', metavar='CC', 
                        default=[], dest='mailcc')
    parser.add_argument('-t', '--mailto', required=True, 
                        action='append', metavar='RECIPIENT',
                        default=[], dest='mailto',
                        help='A To: header value (at least one required)')
    args = parser.parse_args()
    send_mail(subject=args.subject, body=args.body, mailto=args.mailto, mailcc=args.mailcc, attachments=args.attachments, output=args.output)



#!/bin/env python3
# -*- encoding: utf-8 -*-
##############################################################################
#
#    Acrisel LTD
#    Copyright (C) 2008- Acrisel (acrisel.com) . All Rights Reserved
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

import pexpect as expect

def run_abi_cmd(host, cmd, password,):
  p=expect.spawn("ssh " + host)
  i0=p.expect([r'.+password:.*'])
  if i0 == 0:
    p.sendline(password)
    i1=p.expect([r'Sorry.+', r'.*'])
    if i1==0:
      print('Wrongs password')
      return 1
    else:
      print('inside')
      p.sendline(cmd)
  else:
    print('Not expected')
    
if __name__ == '__main__':
  run_abi_cmd("urh12011", "echo I am", 'Chil0542',)
